import pandas as pd
import json
import re

# DXI Label and Metadata

dxi_map = {}

fn = "ICD_10_CM_to_DXI_Mapping_v2021.1.xlsx"
df_dxi = pd.read_excel(fn, sheet_name="ICD_10_CM_to_DXI_Mapping", 
                        engine="openpyxl")
for _, row in df_dxi.iterrows():
    dx = row["ICD10CM"]
    if dx not in dxi_map:
        dxi_map[dx] = {"labels": []}

    label_keys = ["CH_ABBREV", 
                "DXI_1", "DXI_2", "DXI_3", "DXI_4", 
                "DXI_5", "DXI_6", "DXI_7", 
                "Continuous_Var", 
                "HIER_1", "HIER_2", "HIER_3", 
                "HIER_4", "HIER_5", "HIER_6"]
    dxi_map[dx]["labels"] = [row[label_key].strip() 
                                for label_key in label_keys
                                if (not pd.isna(row[label_key]) and 
                                    row[label_key].strip() != "")]
    dxi_map[dx]["dxi_valid_dx"] = row["ICD10CM_VALID"]
    if not pd.isna(row["Continuous_Var_Value"]):
        dxi_map[dx]["cont_value"] = row["Continuous_Var_Value"]


fn = "DXCCSR-Reference-File-v2023-1.xlsx"
df_ccsr = pd.read_excel(fn, sheet_name="DX_to_CCSR_Mapping", skiprows=1,
                        engine="openpyxl")
for _, row in df_ccsr.iterrows():
    dx = row["ICD-10-CM Code"]
    if dx not in dxi_map:
        dxi_map[dx] = {"labels": []}
    dxi_map[dx]["labels"].append("CCSR_" + row["CCSR Category"])
    dxi_map[dx]["ccsr_ip_ynx"] = row["Inpatient Default CCSR (Y/N/X)"]
    dxi_map[dx]["ccsr_op_ynx"] = row["Outpatient Default CCSR (Y/N/X)"]

with open("dxi_mapping.json", "w") as fp:
    json.dump(dxi_map, fp=fp, indent=2)






import json
import csv
import re

class DXIEngine:

    def __init__(self):

        fn_mapping = "src/dxitemspy/dxi_mapping.json"
        fn_metadata = "src/dxitemspy/dxi_metadata.json"
        self.dxi_mapping = {}
        self.dxi_metadata = {}
        with open(fn_mapping, "r") as fp:
            self.dxi_mapping = json.load(fp)
        with open(fn_metadata, "r") as fp:
            self.dxi_metadata = json.load(fp)

    def get_dxi(self, dx_lst):

        dx2dxi = {} 
        labels = []
        for dx in set(dx_lst):
            if dx in self.dxi_mapping:
                dx2dxi[dx] = self.dxi_mapping[dx]["labels"][:]
                labels += dx2dxi[dx]
        
        labels = list(set(labels))
        lb2meta = {}
        score = 0.0
        for lb in labels:
            if lb in self.dxi_metadata:
                lb2meta[lb] = self.dxi_metadata[lb]
                if lb2meta[lb].get("has_default_coeff", False):
                    score = lb2meta[lb].get("score", 0)

        out = {"labels": labels,
                "labels_info": lb2meta,
                "dx2dxi": dx2dxi,
                "score": score}

        return out






import pandas as pd
import json
import re

# DXI Label and Metadata

fn = "DXI_Labels_v2021.1.xlsx"
df_dxi1 = pd.read_excel(fn, sheet_name="DXI_1_Labels", engine="openpyxl")
df_dxi2 = pd.read_excel(fn, sheet_name="DXI_2_Labels", engine="openpyxl")
df_dxi3 = pd.read_excel(fn, sheet_name="Continuous_Var_Labels", 
                        engine="openpyxl")
df_hier = pd.read_excel(fn, sheet_name="Hier_Labels", engine="openpyxl")


dxi = {}
for _, row in df_dxi1.iterrows():
    dxi[row["DXI_1"]] = {"label": row["DXI_LABEL_1"], 
                        "ch_abbrev": row["CH_ABBREV"],
                        "system": "DXI",
                        "level": 1}
for _, row in df_dxi2.iterrows():
    dxi[row["DXI_2"]] = {"label": row["DXI_LABEL_2"],
                        "ch_abbrev": row["CH_ABBREV"],
                        "system": "DXI",
                        "level": 2}
for _, row in df_dxi3.iterrows():
    dxi[row["CONTINUOUS_VAR"]] = {"label": row["CONTINUOUS_VAR_LABEL"],
                            "cont_value": row["CONTINUOUS_VAR_VALUE"],
                            "cont_format": row["CONTINUOUS_VAR_FORMAT"],
                            "ch_abbrev": row["CH_ABBREV"],
                            "system": "DXI",
                            "level": 3}
for _, row in df_hier.iterrows():
    dxi[row["HIER"]] = {"label": row["HIER_LABEL"],
                        "ch_abbrev": row["CH_ABBREV"],
                        "level": 0}

fn = "DXCCSR-Reference-File-v2023-1.xlsx"
df_ccsr = pd.read_excel(fn, sheet_name="CCSR_Categories", skiprows=1,
                        engine="openpyxl")
for _, row in df_ccsr.iterrows():
    dxi["CCSR_"+row["CCSR Category"]] = {
                            "label": row["CCSR Category Description"],
                            "ch_abbrev": "CCSR",
                            "system": "CCSR",
                            "level": 1}

fn = "Z_DXI_Model1_P_AnnTotalSpend_250K.sas"
with open(fn, "r") as fp:
    for line in fp.readlines():
        pattern = "(\w+)\s+\*\s+([\-\d.]+)\s+\+"
        res = re.findall(pattern, line)
        if len(res) > 0 and res[0][0] in dxi:
            dxi[res[0][0]]["has_default_coeff"] = True
            dxi[res[0][0]]["score"] = float(res[0][1])

with open("dxi_metadata.json", "w") as fp:
    json.dump(dxi, fp=fp, indent=2)






import unittest
from dxitemspy.dxitems import DXIEngine


class TestDXIEngine(unittest.TestCase):

    def test_dxipy(self):
        de = DXIEngine()

        out = de.get_dxi(["A00"])
        self.assertTrue("INF" in out["labels"])

        out = de.get_dxi(["I10"])
        self.assertTrue("DXI1_CIR002" in out["labels"])

        out = de.get_dxi(["A00", "I10"])
        self.assertTrue(len(out["labels"])==7)



